@@ -19,9 +19,11 @@ pub use builder::{logger, Builder, PendingValidation, Validated};
 pub(crate) use health::Health;
 pub(crate) use metrics::Metrics;
 pub use server::Server;
+pub use sessions::SessionKey;
 
 mod admin;
 mod builder;
+mod config_dump;
 mod health;
 mod metrics;
 mod server;
